﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.Globalization;          // added to include serial ports
using System.Drawing.Imaging;

namespace tpa64
{
    public partial class Form1 : Form
    {
        static SerialPort USB_PORT;
        byte[] SerBuf = new byte[200];
        float[,] temperature_array = new float[64, 5];
        float[] filtered_temperatures= new float[64];
        byte pointer = 0;
        Bitmap bm = new Bitmap(8, 8);

        public Form1()
        {
            InitializeComponent();
            USB_PORT = new SerialPort();

            foreach (string s in SerialPort.GetPortNames())        // Get a list of available serial port names.
            {
                comboBox_comport.Items.Add(s);                      // places each "COMx" name into combobox
            }
            comboBox_comport.Text = "Select COM Port";              // printed only at first execution in the combobox
        }

        private void comboBox_comport_SelectedIndexChanged(object sender, EventArgs e)
        {
            USB_PORT.Close();                            // close any existing handle
            USB_PORT.PortName = comboBox_comport.Text;    // retrieves "COMx" from selection in combo box
            USB_PORT.ReadTimeout = 50;
            USB_PORT.WriteTimeout = 50;
            USB_PORT.Open();

            SerBuf[0] = 0x5A;       // get version
            Write(1);
            Read(2);

            if (SerBuf[0] == 37)  // if the module id is that of the USB-ISS 
            {
                Version.Text = string.Format("USB-TPA64 V{0}", SerBuf[1]);  //print the software version on screen
                timer1.Enabled = true;
                label5.Visible = label6.Visible = label7.Visible = true;
            }
            else Version.Text = "Not Found";
        }

        private void Write(int write_bytes)
        {
            try
            {
                USB_PORT.Write(SerBuf, 0, write_bytes);      // writes specified amount of SerBuf out on COM port
            }
            catch (Exception)
            {
            }
        }

        private void Read(int read_bytes)
        {
            byte x;

            for (x = 0; x < read_bytes; x++)       // this will call the read function for the passed number times, 
            {                                      // this way it ensures each byte has been correctly recieved while
                try                                // still using timeouts
                {
                    USB_PORT.Read(SerBuf, x, 1);     // retrieves 1 byte at a time and places in SerBuf at position x
                }
                catch (Exception)                   // timeout or other error occured, set lost comms indicator
                {
                    SerBuf[0] = 255;
                }
            }
        }


        private void timer1_Tick(object sender, EventArgs e)
        {
            int x, y;
            float ambient, temp, thermmax, thermmin;
            float intensity;
            Bitmap Palette = new Bitmap(usbtpa64.Properties.Resources.intensity_mask); //a 256 x 1 bmp with a gradient, this acts as a lookup for colour reletive to temperature
            //retrieve temperatures from AMG8833, placing integer results in ambient (for onboard themistor) and 128 bytes into our temperature array (64 temperature integers)
            // the multidimensional temperature array is 128 x 5, thats 64 integer temperatures (2 bytes each) and 5 sets of results for filtering

            SerBuf[0] = 0x5B;
            Write(1);
            Read(130);

            ambient = Convert.ToInt16(((SerBuf[0x01] &0x07) << 8) + SerBuf[0x00]);
            ambient *= 0.0625F;
            if ((SerBuf[0x01] & 0x08) == 0x08) ambient = 0 - ambient;
            textBox_temp.Text = ambient.ToString() + "°C";

            for (x = 2; x < 130; x += 2)
            {
                if(SerBuf[x+1] < 8) temperature_array[(x - 2) / 2, pointer] = (float)(((SerBuf[x + 1] << 8) + SerBuf[x]) * 0.25); //+ve temperatures
                else temperature_array[(x - 2) / 2, pointer] =  (float) ((0x1000 -((SerBuf[x + 1] << 8) + SerBuf[x])) * 0.25);//-ve temperatures
            }

            //averaging
            for (x = 0; x < 64; x++)
            {
                for (temp = 0, y = 0; y < 5; y++) temp += temperature_array[x, y];
                filtered_temperatures[x] = temp / 5;
            }
            pointer++;
            if (pointer > 4) pointer = 0;

            //set maximum and minimum temperature values
            thermmax = -200;
            thermmin = 200;
            for (x = 0; x < 64; x++)
            {
                if (filtered_temperatures[x] > thermmax) thermmax = filtered_temperatures[x];
                else if (filtered_temperatures[x] < thermmin) thermmin = filtered_temperatures[x];
            }
            textBox_thermmax.Text = thermmax.ToString() + "°C";
            textBox_thermmin.Text = thermmin.ToString() + "°C";

            
            System.Drawing.SolidBrush myBrush = new System.Drawing.SolidBrush(System.Drawing.Color.Green);
            System.Drawing.Graphics formGraphics;
            formGraphics = this.CreateGraphics();

            //raw data image, 256x256
            for (y = 0; y < 8; y++)
            {
                for (x = 0; x < 8; x++)
                {
                    temp = filtered_temperatures[x + (y * 8)];
                    if (temp > ambient + 1) //filter out noise, temperature of pixel has to be 1° above ambient
                    {
                        intensity = ((temp - thermmin) / (thermmax - thermmin));
                        if (intensity < 0) intensity = 0;
                        myBrush.Color = Color.FromArgb((int)(255f * intensity), (int)(255f * (1 - intensity)), 0);
                    }
                    else myBrush.Color = Color.Green;
                    formGraphics.FillRectangle(myBrush, new Rectangle(20 + (x * 32), 256 - (y * 32), 32, 32));
                    System.Drawing.SolidBrush drawBrush = new System.Drawing.SolidBrush(System.Drawing.Color.Black);
                    System.Drawing.StringFormat drawFormat = new System.Drawing.StringFormat();
                    System.Drawing.Font drawFont = new System.Drawing.Font("Arial", 8);
                    formGraphics.DrawString(temp.ToString("0.#"), drawFont, drawBrush, 20 + (x * 32), 256 - (y * 32), drawFormat);
                }

            }

            //creates a 8x8 image (not displayed) using the gradient pallete (intensity-mask.bmp) relative to temperature
            for (y = 0; y < 8; y++)
            {
                for (x = 0; x < 8; x++)
                {
                    temp = filtered_temperatures[x + (y * 8)];
                    if (thermmax - thermmin < 10) intensity = ((temp - thermmin) / 10); // the 256 colour pallete is mapped over the temperature range, make sure we map over at least 10°.  
                    else intensity = ((temp - thermmin) / (thermmax - thermmin));   //range over 10°
                    if (intensity < 0) intensity = 0;
                    myBrush.Color = Palette.GetPixel((int)(intensity * 255), 0);

                    using (Graphics gr = Graphics.FromImage(bm))
                    {
                        gr.FillRectangle(myBrush, new Rectangle((x * 1), 8 - (1 * y), 1, 1));
                    }
                }
            }

            //now use the small 8x8 image to create a 256x256 interpolated image by using C# library functions 
            Size size = new Size(256, 256);
            Image a = new Bitmap(size.Width, size.Height);
            Image b = new Bitmap(size.Width, size.Height);

            using (Graphics g = Graphics.FromImage(a))
            {
                g.SmoothingMode = SmoothingMode.HighQuality;
                g.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.HighQuality;
                g.InterpolationMode = InterpolationMode.HighQualityBilinear;
                g.DrawImage(bm, new Rectangle(Point.Empty, size));
                formGraphics.DrawImage(a, 20, 340);
            }            
            

            ImageAttributes wrapMode = new ImageAttributes();
            wrapMode.SetWrapMode(WrapMode.TileFlipXY);      //this helps stop the bordering pixels being evaluated, when the filter hits the edge of the image it will use the other sides pixels
            using (Graphics g = Graphics.FromImage(b))
            {
                g.SmoothingMode = SmoothingMode.HighQuality;
                g.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.HighQuality;
                g.InterpolationMode = InterpolationMode.HighQualityBicubic;
                g.DrawImage(bm, new Rectangle(Point.Empty, size), 0, 0, bm.Width, bm.Height, GraphicsUnit.Pixel, wrapMode);
                formGraphics.DrawImage(b, 320, 340);
            }

            myBrush.Dispose();
            formGraphics.Dispose();
        }
    }
}
